@@ -11,12 +11,7 @@ import (
 
 func containerGet(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
-	//cId, err := dbGetContainerID(d.db, name)  will need cId to get info
-	_, err := dbGetContainerID(d.db, name)
-	if err != nil {
-		return NotFound
-	}
-	c, err := newLxdContainer(name, d)
+	c, err := containerLXDLoad(d, name)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -28,9 +23,7 @@ func containerGet(d *Daemon, r *http.Request) Response {
 
 	targetPath := r.FormValue("log")
 	if strings.ToLower(targetPath) == "true" || targetPath == "1" {
-		fname := c.c.LogFile()
-
-		f, err := os.Open(fname)
+		f, err := os.Open(c.LogFilePathGet())
 		if err != nil {
 			return InternalError(err)
 		}
