@@ -13,20 +13,27 @@ const DB_FIXTURES string = `
     INSERT INTO profiles (name) VALUES ('theprofile');
     INSERT INTO containers_profiles (container_id, profile_id) VALUES (1, 2);
     INSERT INTO containers_config (container_id, key, value) VALUES (1, 'thekey', 'thevalue');
-    INSERT INTO containers_devices (container_id, name) VALUES (1, 'somename');
+    INSERT INTO containers_devices (container_id, name, type) VALUES (1, 'somename', 1);
     INSERT INTO containers_devices_config (key, value, container_device_id) VALUES ('configkey', 'configvalue', 1);
     INSERT INTO images (fingerprint, filename, size, architecture, creation_date, expiry_date, upload_date) VALUES ('fingerprint', 'filename', 1024, 0,  1431547174,  1431547175,  1431547176);
     INSERT INTO images_aliases (name, image_id, description) VALUES ('somealias', 1, 'some description');
     INSERT INTO images_properties (image_id, type, key, value) VALUES (1, 0, 'thekey', 'some value');
     INSERT INTO profiles_config (profile_id, key, value) VALUES (2, 'thekey', 'thevalue');
-    INSERT INTO profiles_devices (profile_id, name) VALUES (2, 'devicename');
+    INSERT INTO profiles_devices (profile_id, name, type) VALUES (2, 'devicename', 1);
     INSERT INTO profiles_devices_config (profile_device_id, key, value) VALUES (2, 'devicekey', 'devicevalue');
     `
 
 //  This Helper will initialize a test in-memory DB.
 func createTestDb(t *testing.T) (db *sql.DB) {
+	// Setup logging if main() hasn't been called/when testing
+	if shared.Log == nil {
+		shared.SetLogger("", "", true, true)
+	}
+
 	var err error
-	db, err = initializeDbObject(":memory:")
+	d := &Daemon{}
+	err = initializeDbObject(d, ":memory:")
+	db = d.db
 
 	if err != nil {
 		t.Fatal(err)
@@ -182,7 +189,10 @@ func Test_initializing_db_is_indempotent(t *testing.T) {
 	var err error
 
 	// This calls "createDb" once already.
-	db, err = initializeDbObject(":memory:")
+	d := &Daemon{}
+	err = initializeDbObject(d, ":memory:")
+	db = d.db
+
 	defer db.Close()
 
 	// Let's call it a second time.
@@ -200,14 +210,14 @@ func Test_get_schema_returns_0_on_uninitialized_db(t *testing.T) {
 	if err != nil {
 		t.Error(err)
 	}
-	result := getSchema(db)
+	result := dbGetSchema(db)
 
 	if result != 0 {
 		t.Error("getSchema should return 0 on uninitialized db!")
 	}
 }
 
-func Test_running_updateFromV6_adds_on_delete_cascade(t *testing.T) {
+func Test_running_dbUpdateFromV6_adds_on_delete_cascade(t *testing.T) {
 	// Upgrading the database schema with updateFromV6 adds ON DELETE CASCADE
 	// to sqlite tables that require it, and conserve the data.
 
@@ -216,7 +226,10 @@ func Test_running_updateFromV6_adds_on_delete_cascade(t *testing.T) {
 	var count int
 	var statements string
 
-	db, err = initializeDbObject(":memory:")
+	d := &Daemon{}
+	err = initializeDbObject(d, ":memory:")
+	db = d.db
+
 	defer db.Close()
 
 	if err != nil {
@@ -250,7 +263,7 @@ INSERT INTO containers_config (container_id, key, value) VALUES (1, 'thekey', 't
 	}
 
 	// Run the upgrade from V6 code
-	err = updateFromV6(db)
+	err = dbUpdateFromV6(db)
 
 	// Make sure the inserted data is still there.
 	statements = `SELECT count(*) FROM containers_config;`
@@ -360,14 +373,16 @@ INSERT INTO containers_config (container_id, key, value) VALUES (1, 'thekey', 't
 
 	// The "foreign key" on containers_config now points to nothing.
 	// Let's run the schema upgrades.
-	err = updateDb(db, 1)
+	d := &Daemon{}
+	d.db = db
+	err = dbUpdate(d, 1)
 
 	if err != nil {
 		t.Error("Error upgrading database schema!")
 		t.Fatal(err)
 	}
 
-	result := getSchema(db)
+	result := dbGetSchema(db)
 	if result != DB_CURRENT_VERSION {
 		t.Fatal(fmt.Sprintf("The schema is not at the latest version after update! Found: %d, should be: %d", result, DB_CURRENT_VERSION))
 	}
@@ -431,7 +446,7 @@ func Test_dbImageGet_for_missing_fingerprint(t *testing.T) {
 	}
 }
 
-func Test_dbAliasGet_alias_exists(t *testing.T) {
+func Test_dbImageAliasGet_alias_exists(t *testing.T) {
 	var db *sql.DB
 	var err error
 	var result string
@@ -439,7 +454,7 @@ func Test_dbAliasGet_alias_exists(t *testing.T) {
 	db = createTestDb(t)
 	defer db.Close()
 
-	result, err = dbAliasGet(db, "somealias")
+	result, err = dbImageAliasGet(db, "somealias")
 
 	if err != nil {
 		t.Fatal(err)
@@ -451,14 +466,14 @@ func Test_dbAliasGet_alias_exists(t *testing.T) {
 
 }
 
-func Test_dbAliasGet_alias_does_not_exists(t *testing.T) {
+func Test_dbImageAliasGet_alias_does_not_exists(t *testing.T) {
 	var db *sql.DB
 	var err error
 
 	db = createTestDb(t)
 	defer db.Close()
 
-	_, err = dbAliasGet(db, "whatever")
+	_, err = dbImageAliasGet(db, "whatever")
 
 	if err != NoSuchObjectError {
 		t.Fatal("Error should be NoSuchObjectError")
@@ -466,7 +481,7 @@ func Test_dbAliasGet_alias_does_not_exists(t *testing.T) {
 
 }
 
-func Test_dbAddAlias(t *testing.T) {
+func Test_dbImageAliasAdd(t *testing.T) {
 	var db *sql.DB
 	var err error
 	var result string
@@ -474,12 +489,12 @@ func Test_dbAddAlias(t *testing.T) {
 	db = createTestDb(t)
 	defer db.Close()
 
-	err = dbAddAlias(db, "Chaosphere", 1, "Someone will like the name")
+	err = dbImageAliasAdd(db, "Chaosphere", 1, "Someone will like the name")
 	if err != nil {
 		t.Fatal("Error inserting Image alias.")
 	}
 
-	result, err = dbAliasGet(db, "Chaosphere")
+	result, err = dbImageAliasGet(db, "Chaosphere")
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -489,7 +504,7 @@ func Test_dbAddAlias(t *testing.T) {
 	}
 }
 
-func Test_dbGetConfig(t *testing.T) {
+func Test_dbContainerConfigGet(t *testing.T) {
 	var db *sql.DB
 	var err error
 	var result map[string]string
@@ -500,7 +515,7 @@ func Test_dbGetConfig(t *testing.T) {
 
 	_, err = db.Exec("INSERT INTO containers_config (container_id, key, value) VALUES (1, 'something', 'something else');")
 
-	result, err = dbGetConfig(db, 1)
+	result, err = dbContainerConfigGet(db, 1)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -514,7 +529,7 @@ func Test_dbGetConfig(t *testing.T) {
 	}
 }
 
-func Test_dbGetProfileConfig(t *testing.T) {
+func Test_dbProfileConfigGet(t *testing.T) {
 	var db *sql.DB
 	var err error
 	var result map[string]string
@@ -525,7 +540,7 @@ func Test_dbGetProfileConfig(t *testing.T) {
 
 	_, err = db.Exec("INSERT INTO profiles_config (profile_id, key, value) VALUES (2, 'something', 'something else');")
 
-	result, err = dbGetProfileConfig(db, "theprofile")
+	result, err = dbProfileConfigGet(db, "theprofile")
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -539,7 +554,7 @@ func Test_dbGetProfileConfig(t *testing.T) {
 	}
 }
 
-func Test_dbGetProfiles(t *testing.T) {
+func Test_dbContainerProfilesGet(t *testing.T) {
 	var db *sql.DB
 	var err error
 	var result []string
@@ -549,7 +564,7 @@ func Test_dbGetProfiles(t *testing.T) {
 	defer db.Close()
 
 	expected = []string{"theprofile"}
-	result, err = dbGetProfiles(db, 1)
+	result, err = dbContainerProfilesGet(db, 1)
 
 	if err != nil {
 		t.Fatal(err)
@@ -562,7 +577,7 @@ func Test_dbGetProfiles(t *testing.T) {
 	}
 }
 
-func Test_dbGEtDevices_profiles(t *testing.T) {
+func Test_dbDevicesGet_profiles(t *testing.T) {
 	var db *sql.DB
 	var err error
 	var result shared.Devices
@@ -572,23 +587,23 @@ func Test_dbGEtDevices_profiles(t *testing.T) {
 	db = createTestDb(t)
 	defer db.Close()
 
-	result, err = dbGetDevices(db, "theprofile", true)
+	result, err = dbDevicesGet(db, "theprofile", true)
 	if err != nil {
 		t.Fatal(err)
 	}
 
-	expected = shared.Device{"type": "0", "devicekey": "devicevalue"}
+	expected = shared.Device{"type": "nic", "devicekey": "devicevalue"}
 	subresult = result["devicename"]
 
 	for key, value := range expected {
 		if subresult[key] != value {
-			t.Errorf("Mismatching value for key %s: %s != %s", key, result[key], value)
+			t.Errorf("Mismatching value for key %s: %v != %v", key, subresult[key], value)
 		}
 	}
 
 }
 
-func Test_dbGEtDevices_containers(t *testing.T) {
+func Test_dbDevicesGet_containers(t *testing.T) {
 	var db *sql.DB
 	var err error
 	var result shared.Devices
@@ -598,17 +613,17 @@ func Test_dbGEtDevices_containers(t *testing.T) {
 	db = createTestDb(t)
 	defer db.Close()
 
-	result, err = dbGetDevices(db, "thename", false)
+	result, err = dbDevicesGet(db, "thename", false)
 	if err != nil {
 		t.Fatal(err)
 	}
 
-	expected = shared.Device{"type": "0", "configkey": "configvalue"}
+	expected = shared.Device{"type": "nic", "configkey": "configvalue"}
 	subresult = result["somename"]
 
 	for key, value := range expected {
 		if subresult[key] != value {
-			t.Errorf("Mismatching value for key %s: %s != %s", key, result[key], value)
+			t.Errorf("Mismatching value for key %s: %s != %s", key, subresult[key], value)
 		}
 	}
 
