@@ -11,7 +11,6 @@ import (
 	"regexp"
 	"strconv"
 	"strings"
-	"syscall"
 	"unsafe"
 
 	"github.com/gorilla/mux"
@@ -27,10 +26,11 @@ func socketPath() string {
 type DevLxdResponse struct {
 	content interface{}
 	code    int
+	ctype   string
 }
 
-func OkResponse(ct interface{}) *DevLxdResponse {
-	return &DevLxdResponse{ct, http.StatusOK}
+func OkResponse(ct interface{}, ctype string) *DevLxdResponse {
+	return &DevLxdResponse{ct, http.StatusOK, ctype}
 }
 
 type DevLxdHandler struct {
@@ -42,43 +42,44 @@ type DevLxdHandler struct {
 	 * server side right now either, I went the simple route to avoid
 	 * needless noise.
 	 */
-	f func(c *lxdContainer, r *http.Request) *DevLxdResponse
+	f func(c container, r *http.Request) *DevLxdResponse
 }
 
-var configGet = DevLxdHandler{"/1.0/config", func(c *lxdContainer, r *http.Request) *DevLxdResponse {
-	filtered := map[string]string{}
-	for k, v := range c.config {
+var configGet = DevLxdHandler{"/1.0/config", func(c container, r *http.Request) *DevLxdResponse {
+	filtered := []string{}
+	for k, _ := range c.ConfigGet().Config {
 		if strings.HasPrefix(k, "user.") {
-			filtered[k] = v
+			filtered = append(filtered, fmt.Sprintf("/1.0/config/%s", k))
 		}
 	}
-	return OkResponse(filtered)
+	return OkResponse(filtered, "json")
 }}
 
-var configKeyGet = DevLxdHandler{"/1.0/config/{key}", func(c *lxdContainer, r *http.Request) *DevLxdResponse {
+var configKeyGet = DevLxdHandler{"/1.0/config/{key}", func(c container, r *http.Request) *DevLxdResponse {
 	key := mux.Vars(r)["key"]
 	if !strings.HasPrefix(key, "user.") {
-		return &DevLxdResponse{"not authorized", http.StatusForbidden}
+		return &DevLxdResponse{"not authorized", http.StatusForbidden, "raw"}
 	}
 
-	value, ok := c.config[key]
+	value, ok := c.ConfigGet().Config[key]
 	if !ok {
-		return &DevLxdResponse{"not found", http.StatusNotFound}
+		return &DevLxdResponse{"not found", http.StatusNotFound, "raw"}
 	}
 
-	return OkResponse(value)
+	return OkResponse(value, "raw")
 }}
 
-var metadataGet = DevLxdHandler{"/1.0/meta-data", func(c *lxdContainer, r *http.Request) *DevLxdResponse {
-	return &DevLxdResponse{"not implemented", http.StatusNotImplemented}
+var metadataGet = DevLxdHandler{"/1.0/meta-data", func(c container, r *http.Request) *DevLxdResponse {
+	value := c.ConfigGet().Config["user.meta-data"]
+	return OkResponse(fmt.Sprintf("#cloud-config\ninstance-id: %s\nlocal-hostname: %s\n%s", c.NameGet(), c.NameGet(), value), "raw")
 }}
 
 var handlers = []DevLxdHandler{
-	DevLxdHandler{"/", func(c *lxdContainer, r *http.Request) *DevLxdResponse {
-		return OkResponse([]string{"/1.0"})
+	DevLxdHandler{"/", func(c container, r *http.Request) *DevLxdResponse {
+		return OkResponse([]string{"/1.0"}, "json")
 	}},
-	DevLxdHandler{"/1.0", func(c *lxdContainer, r *http.Request) *DevLxdResponse {
-		return OkResponse(shared.Jmap{"api_compat": 0})
+	DevLxdHandler{"/1.0", func(c container, r *http.Request) *DevLxdResponse {
+		return OkResponse(shared.Jmap{"api_compat": 0}, "json")
 	}},
 	configGet,
 	configKeyGet,
@@ -86,7 +87,7 @@ var handlers = []DevLxdHandler{
 	/* TODO: events */
 }
 
-func hoistReq(f func(*lxdContainer, *http.Request) *DevLxdResponse, d *Daemon) func(http.ResponseWriter, *http.Request) {
+func hoistReq(f func(container, *http.Request) *DevLxdResponse, d *Daemon) func(http.ResponseWriter, *http.Request) {
 	return func(w http.ResponseWriter, r *http.Request) {
 		conn := extractUnderlyingConn(w)
 		pid, ok := pidMapper.m[conn]
@@ -103,20 +104,42 @@ func hoistReq(f func(*lxdContainer, *http.Request) *DevLxdResponse, d *Daemon) f
 
 		resp := f(c, r)
 		if resp.code != http.StatusOK {
+			w.Header().Set("Content-Type", "application/octet-stream")
 			http.Error(w, fmt.Sprintf("%s", resp.content), resp.code)
+		} else if resp.ctype == "json" {
+			w.Header().Set("Content-Type", "application/json")
+			WriteJSON(w, resp.content)
 		} else {
-			WriteJson(w, resp.content)
+			w.Header().Set("Content-Type", "application/octet-stream")
+			fmt.Fprintf(w, resp.content.(string))
 		}
 	}
 }
 
 func createAndBindDevLxd() (*net.UnixListener, error) {
-
 	if err := os.MkdirAll(socketPath(), 0777); err != nil {
 		return nil, err
 	}
 
 	sockFile := path.Join(socketPath(), "sock")
+
+	/*
+	 * If this socket exists, that means a previous lxd died and didn't
+	 * clean up after itself. We assume that the LXD is actually dead if we
+	 * get this far, since StartDaemon() tries to connect to the actual lxd
+	 * socket to make sure that it is actually dead. So, it is safe to
+	 * remove it here without any checks.
+	 *
+	 * Also, it would be nice to SO_REUSEADDR here so we don't have to
+	 * delete the socket, but we can't:
+	 *   http://stackoverflow.com/questions/15716302/so-reuseaddr-and-af-unix
+	 *
+	 * Note that this will force clients to reconnect when LXD is restarted.
+	 */
+	if err := os.Remove(sockFile); err != nil && !os.IsNotExist(err) {
+		return nil, err
+	}
+
 	unixAddr, err := net.ResolveUnixAddr("unix", sockFile)
 	if err != nil {
 		return nil, err
@@ -219,7 +242,6 @@ func (m *ConnPidMapper) ConnStateHandler(conn net.Conn, state http.ConnState) {
  * we need it to get at SO_PEERCRED, so let's grab it.
  */
 func extractUnderlyingFd(unixConnPtr *net.UnixConn) int {
-
 	conn := reflect.Indirect(reflect.ValueOf(unixConnPtr))
 	netFdPtr := conn.FieldByName("fd")
 	netFd := reflect.Indirect(netFdPtr)
@@ -229,12 +251,13 @@ func extractUnderlyingFd(unixConnPtr *net.UnixConn) int {
 
 func getPid(conn *net.UnixConn) (int32, error) {
 	fd := extractUnderlyingFd(conn)
-	cred, err := syscall.GetsockoptUcred(fd, syscall.SOL_SOCKET, syscall.SO_PEERCRED)
+
+	_, _, pid, err := getUcred(fd)
 	if err != nil {
-		return -1, err
+		return 0, err
 	}
 
-	return cred.Pid, nil
+	return pid, nil
 }
 
 /*
@@ -260,8 +283,7 @@ func extractUnderlyingConn(w http.ResponseWriter) *net.UnixConn {
 
 var pidNotInContainerErr = fmt.Errorf("pid not in container?")
 
-func findContainerForPid(pid int32, d *Daemon) (*lxdContainer, error) {
-
+func findContainerForPid(pid int32, d *Daemon) (container, error) {
 	/*
 	 * Try and figure out which container a pid is in. There is probably a
 	 * better way to do this. Based on rharper's initial performance
@@ -292,7 +314,7 @@ func findContainerForPid(pid int32, d *Daemon) (*lxdContainer, error) {
 			parts := strings.Split(string(cmdline), " ")
 			name := parts[len(parts)-1]
 
-			return newLxdContainer(name, d)
+			return containerLXDLoad(d, name)
 		}
 
 		status, err := ioutil.ReadFile(fmt.Sprintf("/proc/%d/status", pid))
@@ -320,18 +342,22 @@ func findContainerForPid(pid int32, d *Daemon) (*lxdContainer, error) {
 		return nil, err
 	}
 
-	containers, err := dbListContainers(d)
+	containers, err := dbContainersList(d.db, cTypeRegular)
 	if err != nil {
 		return nil, err
 	}
 
 	for _, container := range containers {
-		c, err := newLxdContainer(container, d)
+		c, err := containerLXDLoad(d, container)
 		if err != nil {
 			return nil, err
 		}
 
-		pidNs, err := os.Readlink(fmt.Sprintf("/proc/%d/ns/pid", c.c.InitPid()))
+		initpid, err := c.InitPidGet()
+		if err != nil {
+			return nil, err
+		}
+		pidNs, err := os.Readlink(fmt.Sprintf("/proc/%d/ns/pid", initpid))
 		if err != nil {
 			return nil, err
 		}
