@@ -18,6 +18,7 @@ test_config_profiles() {
   lxc config set foo raw.lxc "lxc.notaconfigkey = invalid" && bad=1 || true
   if [ "$bad" -eq 1 ]; then
     echo "allowed setting a bad config value"
+    false
   fi
 
   lxc config device add foo home disk source=/mnt path=/mnt readonly=true
@@ -35,6 +36,37 @@ test_config_profiles() {
   lxc profile device list onenic | grep eth0
   lxc profile device show onenic | grep lxcbr0
 
+  if [ -z "$TRAVIS_PULL_REQUEST" ]; then
+    # test live-adding a nic
+    lxc start foo
+    lxc config device add foo eth2 nic nictype=bridged parent=lxcbr0 name=eth10
+    lxc exec foo -- /sbin/ifconfig -a | grep eth10
+    lxc config device list foo | grep eth2
+    lxc config device remove foo eth2
+
+    # test live-adding a disk
+    lxc config device add foo etc disk source=/etc path=/mnt2 readonly=true
+    lxc exec foo -- ls /mnt2/hosts
+    lxc stop foo
+    lxc start foo
+    lxc exec foo -- ls /mnt2/hosts
+    lxc config device remove foo etc
+    bad=0
+    lxc exec foo -- ls /mnt2/hosts && bad=1 || true
+    if [ "$bad" -eq 1 ]; then
+      echo "disk was not hot-unplugged"
+      false
+    fi
+    lxc stop foo
+    lxc start foo
+    lxc exec foo -- ls /mnt2/hosts && bad=1 || true
+    if [ "$bad" -eq 1 ]; then
+      echo "disk device re-appeared after stop and start"
+      false
+    fi
+    lxc stop foo
+  fi
+
   lxc config set foo user.prop value
   lxc list user.prop=value | grep foo
   lxc config unset foo user.prop
