@@ -74,9 +74,16 @@ wipe() {
     rm -Rf "$1"
 }
 
+curtest=setup
+
 cleanup() {
     set +x
 
+    echo "==> Test result: $RESULT"
+    if [ $RESULT != "success" ]; then
+      echo "failed test: $curtest"
+    fi
+
     if [ -n "$LXD_INSPECT" ]; then
         echo "To poke around, use:\n LXD_DIR=$LXD_DIR sudo -E $GOPATH/bin/lxc COMMAND --config ${LXD_CONF}"
         read -p "Tests Completed ($RESULT): hit enter to continue" x
@@ -93,7 +100,7 @@ cleanup() {
     for p in `pidof lxd`; do
         pgrp=`awk '{ print $5 }' /proc/$p/stat`
         if [ "$pgrp" = "$mygrp" ]; then
-            kill -9 $p
+          do_kill_lxd $p
         fi
     done
 
@@ -104,10 +111,21 @@ cleanup() {
     done
 
     rm -f devlxd-client || true
+    find . -name shmounts -exec "umount" "-l" "{}" \; || true
 
     echo ""
     echo ""
     echo "==> Test result: $RESULT"
+    if [ $RESULT != "success" ]; then
+      echo "failed test: $curtest"
+    fi
+}
+
+do_kill_lxd() {
+  pid=$1
+  kill -15 $pid
+  sleep 2
+  kill -9 $pid 2>/dev/null || true
 }
 
 trap cleanup EXIT HUP INT TERM
@@ -150,8 +168,13 @@ spawn_lxd() {
   lxddir=$2
   shift
   shift
+
+  # Copy pre generated Certs
+  cp server.crt $lxddir
+  cp server.key $lxddir
+
   echo "==> Spawning lxd on $addr in $lxddir"
-  (LXD_DIR=$lxddir lxd $debug --tcp $addr $extraargs $* 2>&1 & echo $! > $lxddir/lxd.pid) | tee $lxddir/lxd.log &
+  LXD_DIR=$lxddir lxd --logfile $lxddir/lxd.log $debug $extraargs $* 2>&1 & echo $! > $lxddir/lxd.pid
 
   echo "==> Confirming lxd on $addr is responsive"
   alive=0
@@ -160,6 +183,9 @@ spawn_lxd() {
     sleep 1s
   done
 
+  echo "==> Binding to network"
+  LXD_DIR=$lxddir lxc config set core.https_address $addr
+
   echo "==> Setting trust password"
   LXD_DIR=$lxddir lxc config set core.trust_password foo
   if [ -n "$LXD_DEBUG" ]; then
@@ -167,6 +193,16 @@ spawn_lxd() {
   fi
 }
 
+ensure_import_testimage() {
+  if ! lxc image alias list | grep -q "^| testimage\s*|.*$"; then
+    if [ -e "$LXD_TEST_IMAGE" ]; then
+        lxc image import $LXD_TEST_IMAGE --alias testimage
+    else
+        ../scripts/lxd-images import busybox --alias testimage
+    fi
+  fi
+}
+
 spawn_lxd 127.0.0.1:18443 $LXD_DIR
 
 export LXD2_DIR=$(mktemp -d -p $(pwd))
@@ -181,73 +217,95 @@ if [ "$#" -gt 0 ]; then
 fi
 
 echo "==> TEST: commit sign-off"
+curtest=test_commits_signed_off
 test_commits_signed_off
 
 echo "==> TEST: doing static analysis of commits"
+curtest=static_analysis
 static_analysis
 
 echo "==> TEST: checking dependencies"
+curtest=test_check_deps
 test_check_deps
 
 echo "==> TEST: Database schema update"
+curtest=test_database_update
 test_database_update
 
 echo "==> TEST: lxc remote url"
+curtest=test_remote_url
 test_remote_url
 
 echo "==> TEST: lxc remote administration"
+curtest=test_remote_admin
 test_remote_admin
 
 echo "==> TEST: basic usage"
+curtest=test_basic_usage
 test_basic_usage
 
-echo "==> TEST: concurrent exec"
-test_concurrent_exec
+if [ -n "$LXD_CONCURRENT" ]; then
+    echo "==> TEST: concurrent exec"
+    curtest=test_concurrent_exec
+    test_concurrent_exec
 
-echo "==> TEST: concurrent startup"
-test_concurrent
+    echo "==> TEST: concurrent startup"
+    curtest=test_concurrent
+    test_concurrent
+fi
 
 echo "==> TEST: lxc remote usage"
+curtest=test_remote_usage
 test_remote_usage
 
 echo "==> TEST: snapshots"
+curtest=test_snapshots
 test_snapshots
 
 echo "==> TEST: snapshot restore"
+curtest=test_snap_restore
 test_snap_restore
 
 echo "==> TEST: profiles, devices and configuration"
+curtest=test_config_profiles
 test_config_profiles
 
 echo "==> TEST: server config"
+curtest=test_server_config
 test_server_config
 
 echo "==> TEST: filemanip"
+curtest=test_filemanip
 test_filemanip
 
 echo "==> TEST: devlxd"
+curtest=test_devlxd
 test_devlxd
 
 if type fuidshift >/dev/null 2>&1; then
     echo "==> TEST: uidshift"
+    curtest=test_fuidshift
     test_fuidshift
 else
     echo "==> SKIP: fuidshift (binary missing)"
 fi
 
 echo "==> TEST: migration"
+curtest=test_migration
 test_migration
 
 if [ -n "$TRAVIS_PULL_REQUEST" ]; then
     echo "===> SKIP: lvm backing (no loop device on Travis)"
 else
     echo "==> TEST: lvm backing"
+    curtest=test_lvm
     test_lvm
 fi
 
 curversion=`dpkg -s lxc | awk '/^Version/ { print $2 }'`
 if dpkg --compare-versions "$curversion" gt 1.1.2-0ubuntu3; then
     echo "==> TEST: fdleak"
+    curtest=test_fdleak
     test_fdleak
 else
     # We temporarily skip the fdleak test because a bug in lxc is
@@ -257,12 +315,15 @@ else
 fi
 
 echo "==> TEST: cpu profiling"
+curtest=test_cpu_profiling
 test_cpu_profiling
 echo "==> TEST: memory profiling"
+curtest=test_mem_profiling
 test_mem_profiling
 
 # This should always be run last
 echo "==> TEST: database lock"
+curtest=test_database_lock
 test_database_lock
 
 RESULT=success
