@@ -2,31 +2,30 @@ test_snapshots() {
   lxc init testimage foo
 
   lxc snapshot foo
-  [ -d "$LXD_DIR/lxc/foo/snapshots/snap0" ]
+  [ -d "$LXD_DIR/snapshots/foo/snap0" ]
 
   lxc snapshot foo
-  [ -d "$LXD_DIR/lxc/foo/snapshots/snap1" ]
+  [ -d "$LXD_DIR/snapshots/foo/snap1" ]
 
   lxc snapshot foo tester
-  [ -d "$LXD_DIR/lxc/foo/snapshots/tester" ]
+  [ -d "$LXD_DIR/snapshots/foo/tester" ]
 
   lxc copy foo/tester foosnap1
-  [ -d "$LXD_DIR/lxc/foosnap1/rootfs" ]
+  [ -d "$LXD_DIR/containers/foosnap1/rootfs" ]
 
   lxc delete foo/snap0
-  [ ! -d "$LXD_DIR/lxc/foo/snapshots/snap0" ]
+  [ ! -d "$LXD_DIR/snapshots/foo/snap0" ]
 
-  # no CLI for this, so we use the API directly
+  # no CLI for this, so we use the API directly (rename a snapshot)
   wait_for my_curl -X POST $BASEURL/1.0/containers/foo/snapshots/tester -d "{\"name\":\"tester2\"}"
-  [ ! -d "$LXD_DIR/lxc/foo/snapshots/tester" ]
+  [ ! -d "$LXD_DIR/snapshots/foo/tester" ]
 
-  # no CLI for this, so we use the API directly
-  wait_for my_curl -X DELETE $BASEURL/1.0/containers/foo/snapshots/tester2
-  [ ! -d "$LXD_DIR/lxc/foo/snapshots/tester2" ]
+  lxc delete foo/tester2
+  [ ! -d "$LXD_DIR/snapshots/foo/tester2" ]
 
   lxc delete foo
   lxc delete foosnap1
-  [ ! -d "$LXD_DIR/lxc/foo" ]
+  [ ! -d "$LXD_DIR/containers/foo" ]
 }
 
 test_snap_restore() {
@@ -40,30 +39,30 @@ test_snap_restore() {
   lxc launch testimage bar
 
   ##########################################################
-  # PREPARATION  
+  # PREPARATION
   ##########################################################
 
   ## create some state we will check for when snapshot is restored
 
   ## prepare snap0
-  echo snap0 > state 
+  echo snap0 > state
   lxc file push state bar/root/state
   lxc file push state bar/root/file_only_in_snap0
   lxc stop bar --force
-  mkdir "$LXD_DIR/lxc/bar/rootfs/root/dir_only_in_snap0"
-  cd "$LXD_DIR/lxc/bar/rootfs/root/"
+  mkdir "$LXD_DIR/containers/bar/rootfs/root/dir_only_in_snap0"
+  cd "$LXD_DIR/containers/bar/rootfs/root/"
   ln -s ./file_only_in_snap0 statelink
   cd -
 
   lxc snapshot bar snap0
 
   ## prepare snap1
-  echo snap1 > state 
+  echo snap1 > state
   lxc start bar
   lxc file push state bar/root/state
   lxc file push state bar/root/file_only_in_snap1
   lxc stop bar --force
-  cd "$LXD_DIR/lxc/bar/rootfs/root/"
+  cd "$LXD_DIR/containers/bar/rootfs/root/"
 
   rmdir dir_only_in_snap0
   rm    file_only_in_snap0
@@ -95,7 +94,7 @@ test_snap_restore() {
   # test restore using full snapshot name
   restore_and_compare_fs snap1
 
-  # Check config value in snapshot has been restored   
+  # Check config value in snapshot has been restored
   cpus=$(lxc config get bar limits.cpus)
   echo $cpus
   if [ "$cpus" != "limits.cpus: 1" ]; then
@@ -131,6 +130,6 @@ restore_and_compare_fs() {
   lxc restore bar $1
 
   # Recursive diff of container FS
-  echo "diff -r $LXD_DIR/lxc/bar/rootfs $LXD_DIR/lxc/bar/snapshots/$snap/rootfs"
-  diff -r "$LXD_DIR/lxc/bar/rootfs" "$LXD_DIR/lxc/bar/snapshots/$snap/rootfs"
+  echo "diff -r $LXD_DIR/containers/bar/rootfs $LXD_DIR/snapshots/bar/$snap/rootfs"
+  diff -r "$LXD_DIR/containers/bar/rootfs" "$LXD_DIR/snapshots/bar/$snap/rootfs"
 }
