@@ -65,7 +65,7 @@ cleanup() {
     for p in `pidof lxd`; do
         pgrp=`awk '{ print $5 }' /proc/$p/stat`
         if [ "$pgrp" = "$mygrp" ]; then
-            kill -9 $p
+          do_kill_lxd $p
         fi
     done
 
@@ -95,7 +95,7 @@ spawn_lxd() {
   shift
   shift
   echo "==> Spawning lxd on $addr in $lxddir"
-  LXD_DIR=$lxddir lxd ${DEBUG} --tcp $addr $extraargs $* 2>&1 > $lxddir/lxd.log &
+  LXD_DIR=$lxddir lxd ${DEBUG} $extraargs $* 2>&1 > $lxddir/lxd.log &
 
   echo "==> Confirming lxd on $addr is responsive"
   alive=0
@@ -104,6 +104,9 @@ spawn_lxd() {
     sleep 1s
   done
 
+  echo "==> Binding to network"
+  LXD_DIR=$lxddir lxc config set core.https_address $addr
+
   echo "==> Setting trust password"
   LXD_DIR=$lxddir lxc config set core.trust_password foo
 }
